/********************************************************************\

  Name:         null.c
  Created by:   Stefan Ritt

  Contents:     NULL Device Driver


  Revision history
  -------------------------------------------------------------------
  date         by    modification
  ---------    ---   ------------------------------------------------
  26-MAR-97    SR    created


\********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "midas.h"

/*---- globals -----------------------------------------------------*/

typedef struct {
  char      port[32];
} NULL_SETTINGS;

NULL_SETTINGS null_settings;

#define NULL_SETTINGS_STR "\
Port = STRING : [32] Nowhere\n\
"

static float *array = NULL;
static INT   num_channels = -1;

/*---- device driver routines --------------------------------------*/

INT null_init(HNDLE hKey, INT channels)
{
int   status, size;
HNDLE hDB;

  cm_get_experiment_database(&hDB, NULL);

  /* create NULL settings record */
  status = db_create_record(hDB, hKey, "", NULL_SETTINGS_STR);
  if (status != DB_SUCCESS)
    return FE_ERR_ODB;

  size = sizeof(null_settings);
  db_get_record(hDB, hKey, &null_settings, &size, 0);

  /* initialize driver */
  if (num_channels == -1)
    {
    num_channels = channels;
    array = malloc(sizeof(float) * num_channels);
    }
  else
    {
    num_channels = max(num_channels, channels);
    array = realloc(array, sizeof(float) * num_channels);
    }
  memset(array, 0, sizeof(float) * num_channels);

  return FE_SUCCESS;
}

/*----------------------------------------------------------------------------*/

INT null_exit()
{
  if (array)
    {
    free(array);
    array = NULL;
    }

  return FE_SUCCESS;
}

/*----------------------------------------------------------------------------*/

INT null_set(INT channel, float value)
{
  if (channel < num_channels)
    array[channel] = value;

  return FE_SUCCESS;
}

/*----------------------------------------------------------------------------*/

INT null_set_all(INT channels, float value)
{
INT i;

  for (i=0 ; i<min(num_channels, channels) ; i++)
    array[i] = value;

  return FE_SUCCESS;
}

/*----------------------------------------------------------------------------*/

INT null_get(INT channel, float *pvalue)
{
  if (channel < num_channels)
    *pvalue = array[channel];
  else
    *pvalue = 0.f;

  return FE_SUCCESS;
}

/*----------------------------------------------------------------------------*/

INT null_get_all(INT channels, float *pvalue)
{
int   i;

  for (i=0 ; i<min(num_channels, channels) ; i++)
    pvalue[i] = array[i];

  return FE_SUCCESS;
}

/*---- device driver entry point -----------------------------------*/

INT null(INT cmd, ...)
{
va_list argptr;
HNDLE   hKey;
INT     channel, status;
float   value, *pvalue;

  va_start(argptr, cmd);
  status = FE_SUCCESS;

  switch (cmd)
    {
    case CMD_INIT:
      hKey = va_arg(argptr, HNDLE);
      channel = va_arg(argptr, INT);
      status = null_init(hKey, channel);
      break;

    case CMD_EXIT:
      status = null_exit();
      break;

    case CMD_SET:
      channel = va_arg(argptr, INT);
      value   = (float) va_arg(argptr, double);
      status = null_set(channel, value);
      break;

    case CMD_SET_ALL:
      channel = va_arg(argptr, INT);
      value   = (float) va_arg(argptr, double);
      status = null_set_all(channel, value);
      break;

    case CMD_GET:
      channel = va_arg(argptr, INT);
      pvalue  = va_arg(argptr, float*);
      status = null_get(channel, pvalue);
      break;
    
    case CMD_GET_ALL:
      channel = va_arg(argptr, INT);
      pvalue  = va_arg(argptr, float*);
      status = null_get_all(channel, pvalue);
      break;
    
    default:
      break;
    }

  va_end(argptr);

  return status;
}

/*------------------------------------------------------------------*/
