/********************************************************************\

  Name:         rpc_test.c
  Created by:   Stefan Ritt

  Contents:     Mini RPC client for testing purposes

                This program connects to a server running the MIDAS
                main server. It calles the routine rpc_test on this
                server, which simply doubles some arguments. By
                receiving the proper results, one can check that the
                parameter passing via MIDAS RPCs works correctly.


  Revision history
  ------------------------------------------------------------------
  date        by    modification
  ---------   ---   ------------------------------------------------
  13-JUN-94   SR    created


\********************************************************************/

#include "midas.h"
#include "mrpc.h"
#include <stdio.h>

/*------------------------------------------------------------------*/

#ifdef OS_VXWORKS
rpc_test(char *ahost_name)
#else
main()
#endif
{
char   host_name[30];
INT    status;
BYTE   b;
WORD   w;
INT    i;
float  f;
double d;

#ifdef OS_VXWORKS
  strcpy(host_name, ahost_name);
#else
  printf("Remote host IP address: ");
  gets(host_name);
#endif

  status = cm_connect_experiment(host_name, "", "RPC_TEST", NULL);
  if (status == CM_WRONG_PASSWORD)
    {
    printf("Wrong password\n");
    return 0;
    }
  if (status != CM_SUCCESS)
    {
    printf("Cannot connect to MIDAS server running on %s.\n", host_name);
    return 0;
    }

  f = 3.5f;
  d = 4.5;

  /* rpc_test just doubles numbers */
  rpc_call(0, RPC_TEST, 1, 2, 3l, f, d, &b, &w, &i, &f, &d);

  printf("\nResult should be:  2 4 6 7.0 9.0\n");
  printf("Result is:         %d %d %ld %1.1f %1.1lf\n", b, w, i, f, d);

  printf("\nhit return...");
  b = getchar();

  cm_disconnect_experiment();
  return 1;
}
