/********************************************************************\

  Name:         wwwodb.c
  Created by:   Stefan Ritt

  Contents:     MIDAS online database WWW CGI interface


  Revision history
  ------------------------------------------------------------------
  date        by    modification
  ---------   ---   ------------------------------------------------
  19-FEB-96   SR    created


\********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "midas.h"
#include "msystem.h"

void show_help_page(void);
void show_status_page(HNDLE hDB);

char wwwodb_url[256];
char host_name[256];
char exp_name[256];
  
/*------------------------------------------------------------------*/

void urlDecode(char *p) 
/********************************************************************\
   Decode the given string in-place by expanding %XX escapes
\********************************************************************/
{
char *pD, str[3];
int  i;

  pD = p;
  while (*p) 
    {
    if (*p=='%') 
      {
      /* Escape: next 2 chars are hex representation of the actual character */
      p++;
      if (isxdigit(p[0]) && isxdigit(p[1])) 
        {
        str[0] = p[0];
        str[1] = p[1];
        str[2] = 0;
        sscanf(p, "%02X", &i);
        
        *pD++ = (char) i;
        p += 2;
        }
      else
        *pD++ = '%';
      } 
    else 
      {
      *pD++ = *p++;
      }
    }
   *pD = '\0';
}

void urlEncode(char *ps) 
/********************************************************************\
   Encode the given string in-place by adding %XX escapes
\********************************************************************/
{
char *pd, *p, str[256];

  pd = str;
  p  = ps;
  while (*p) 
    {
    if (strchr(" %&=", *p))
      {
      sprintf(pd, "%%%02X", *p);
      pd += 3;
      p++;
      } 
    else 
      {
      *pd++ = *p++;
      }
    }
   *pd = '\0';
   strcpy(ps, str);
}

/*------------------------------------------------------------------*/

void search_callback(HNDLE hDB, HNDLE hKey, KEY *key, void *info)
{
INT         i, size, status;
char        *search_name;
static char data_str[MAX_ODB_PATH];
static char str1[MAX_ODB_PATH], str2[MAX_ODB_PATH], ref[MAX_ODB_PATH];
char        path[MAX_ODB_PATH], data[10000];

  search_name = (char *) info;

  /* convert strings to uppercase */
  for (i=0 ; key->name[i] ; i++)
    str1[i] = toupper(key->name[i]);
  str1[i] = 0;
  for (i=0 ; key->name[i] ; i++)
    str2[i] = toupper(search_name[i]);
  str2[i] = 0;

  if (strstr(str1, str2) != NULL)
    {
    db_get_path(hDB, hKey, path, MAX_ODB_PATH);
    strcpy(str1, path);
    urlEncode(str1);

    if (key->type == TID_KEY || key->type == TID_LINK)
      {
      /* for keys, don't display data value */
      printf("<tr><td><a href=\"%s?host=%s&exp=%s&path=%s\">%s</a></tr>\n", 
             wwwodb_url, host_name, exp_name, str1, path);
      }
    else
      {
      /* display single value */
      if (key->num_values == 1)
        {
        size = sizeof(data);
        status = db_get_data(hDB, hKey, data, &size, key->type);
        if (status == DB_NO_ACCESS)
          strcpy(data_str, "<no read access>");
        else
          db_sprintf(data_str, data, key->item_size, 0, key->type);

        sprintf(ref, "%s?cmd=modify&host=%s&exp=%s&path=%s", 
                wwwodb_url, host_name, exp_name, str1);

        printf("<tr><td>%s<td><a href=\"%s\">%s</a></tr>\n", path, ref, data_str);
        }
      else
        {
        /* display first value */
        printf("<tr><td rowspan=%d>%s\n", key->num_values, path);

        for (i=0 ; i<key->num_values ; i++)
          {
          size = sizeof(data);
          db_get_data(hDB, hKey, data, &size, key->type);
          db_sprintf(data_str, data, key->item_size, i, key->type);

          sprintf(ref, "%s?cmd=modify&index=%d&host=%s&exp=%s&path=%s", 
                  wwwodb_url, i, host_name, exp_name, str1);

          if (i>0)
            printf("<tr>");

          printf("<td><a href=\"%s\">[%d] %s</a></tr>\n", ref, i, data_str);
          }
        }
      }
    }
}

/*------------------------------------------------------------------*/

void interprete(char param[6][128])
/********************************************************************\

  Routine: interprete

  Purpose: Interprete parameter array and generate HTML output from
           odb.

  Input:
    char param              Parameter array
           param[0]         ODB path "host:experiment/dir/subdir/key"
           param[1]         Command "start/stop/find/help"
           param[2]         Value for "set" and "find" command
           param[3]         Index for arrays

\********************************************************************/
{
int   i, j, status, size, index;
HNDLE hKeyRoot, hKey, hDB;
KEY   key;
char  str[256], path[256], url_path[256];
char  ref[256], data_str[256], enc_path[256], dec_path[256];
char  *p, *pd;
char  data[10000];

  /* extract last key from path */
  str[0] = 0;
  if (strlen(param[0]) > 1)
    {
    p = param[0]+strlen(param[0])-1;
    while (*p && *p != '/')
      p--;
    strcpy(str, p+1);
    }

  /* encode path for further usage */
  strcpy(dec_path, param[0]);
  urlDecode(dec_path);
  urlDecode(dec_path); /* necessary for %2520 -> %20 -> ' ' */
  strcpy(enc_path, dec_path);
  urlEncode(enc_path);

  /*---- help command ----------------------------------------------*/
  if (strcmp(param[1], "Help") == 0)
    {
    show_help_page();
    return;
    }
  
  /*---- HTML header -----------------------------------------------*/
  puts("<head>");
  if (str[0])
    printf("<title>MIDAS ODB [%s]</title>", str);
  else
    printf("<title>MIDAS ODB [/]</title>");
  puts("</head>");
  puts("\n<body>");
  printf("<form method=\"GET\" action=\"%s\">\n\n", wwwodb_url);

  /* connect to experiment */
  status = cm_connect_experiment(host_name, exp_name, "WWWOdb", NULL);
  if (status == CM_WRONG_PASSWORD)
    {
    printf("Wrong password\n");
    goto error;
    }
  if (status != CM_SUCCESS)
    {
    printf("Error: cannot connect to experiment %s on host %s (status=%d)<p>\n", 
            exp_name, host_name, status);
    goto error;
    }

  cm_get_experiment_database(&hDB, &hKey);

  /*---- modify command --------------------------------------------*/
  if (strcmp(param[1], "modify") == 0)
    {
    status = db_find_key(hDB, 0, param[0], &hKey);
    if (status != DB_SUCCESS)
      {
      printf("Error: cannot find key %s<P>\n", param[0]);
      goto error;
      }
    db_get_key(hDB, hKey, &key);

    /* define hidden fields for current path etc. */
    printf("<input type=hidden name=host value=%s>\n", host_name);
    printf("<input type=hidden name=exp value=\"%s\">\n", exp_name);
    printf("<input type=hidden name=path value=\"%s\">\n", enc_path);
    printf("<input type=hidden name=cmd value=Set>\n");
    if (host_name[0])
      {
      index = atoi(param[3]);
      printf("<input type=hidden name=index value=\"%d\">\n", index);
      }
    else
      index = 0;

    puts("<table border cellpadding=5>");
    strcpy(data_str, rpc_tid_name(key.type));
    if (key.num_values > 1)
      {
      sprintf(str, "[%d]", key.num_values);
      strcat(data_str, str);
  
      sprintf(str, "%s[%s]", dec_path, param[3]);
      }
    else
      strcpy(str, dec_path);

    printf("<tr><th colspan=2>Set new value - type = %s</tr>\n", data_str);
    printf("<tr><td width=300>%s<td width=200>\n", str);

    /* set current value as default */
    size = sizeof(data);
    db_get_data(hDB, hKey, data, &size, key.type);
    db_sprintf(data_str, data, key.item_size, index, key.type);
    printf("<input type=\"text\" size=\"20\" maxlength=\"80\" name=\"value\" value=\"%s\">\n",
            data_str);
    puts("</tr>");

    puts("<tr><td align=center colspan=2>");
    puts("<input type=submit name=cmd value=Set>");
    puts("<input type=submit name=cmd value=Cancel>");
    puts("</tr>");
    puts("</table>");

    goto exit;
    }

  /*---- start dialog --------------------------------------------*/

  if (strcmp(param[1], "Start") == 0)
    {
    /* check if run is already started */
    size = sizeof(i);
    db_get_value(hDB, 0, "/Runinfo/State", &i, &size, TID_INT);
    if (i == STATE_RUNNING)
      {
      printf("<h1>Run is already started</h1><p>\n");
      }
    else
      {
      if (param[2][0] == 0)
        {
        /* display dialog if no run number is given */

        /* define hidden fields for current path etc. */
        printf("<input type=hidden name=host value=%s>\n", host_name);
        printf("<input type=hidden name=exp value=\"%s\">\n", exp_name);
        printf("<input type=hidden name=path value=\"%s\">\n", enc_path);
        printf("<input type=hidden name=cmd value=Start\n>");

        puts("<table border cellpadding=5>");

        printf("<tr><th colspan=2>Start new run</tr>\n");
        printf("<tr><td>Enter run number\n");

        size = sizeof(i);
        db_get_value(hDB, 0, "/Runinfo/Run number", &i, &size, TID_INT);
    
        printf("<td><input type=text size=20 maxlength=80 name=value value=%d>\n", i+1);
        puts("</tr>");

        puts("<tr><td align=center colspan=2>");
        puts("<input type=submit name=cmd value=Start>");
        puts("<input type=submit name=cmd value=Cancel>");
        puts("</tr>");
        puts("</table>");

        goto exit;
        }
      else
        {
        i = atoi(param[2]);
        printf("<h1>Starting run %d ... \n", i);

        status = cm_transition(TR_START, i, str, sizeof(str), SYNC);
        if (status == CM_SUCCESS)
          printf("OK");
        else
          printf("<br>Error: %s", str);

        printf("</h1><p>\n");
        }
      }
    }

  /*---- stop run --------------------------------------------*/

  if (strcmp(param[1], "Stop") == 0)
    {
    /* check if run is already started */
    size = sizeof(i);
    db_get_value(hDB, 0, "/Runinfo/State", &i, &size, TID_INT);
    if (i == STATE_STOPPED)
      {
      printf("<h1>Run is already stopped</h1><p>\n");
      }
    else
      {
      i = atoi(param[2]);
      printf("<h1>Stopping run ... \n");

      status = cm_transition(TR_STOP, 0, str, sizeof(str), SYNC);
      if (status == CM_SUCCESS)
        printf("OK");
      else
        printf("<br>Error: %s", str);

      printf("</h1><p>\n");
      }
    }

  /*---- search command --------------------------------------------*/
  
  if (strcmp(param[1], "Search") == 0)
    {

    /* define hidden fields for current path etc. */
    printf("<input type=hidden name=host value=%s>\n", host_name);
    printf("<input type=hidden name=exp value=\"%s\">\n", exp_name);
    printf("<input type=hidden name=path value=\"%s\">\n", enc_path);
    printf("<input type=hidden name=cmd value=Find\n>");

    puts("<table border cellpadding=5>");

    printf("<tr><th colspan=2>Search string in Online Database</tr>\n");
    printf("<tr><td>Enter substring (case insensitive)\n");

    printf("<td><input type=\"text\" size=\"20\" maxlength=\"80\" name=\"value\">\n");
    puts("</tr>");

    puts("<tr><td align=center colspan=2>");
    puts("<input type=submit name=cmd value=Find>");
    puts("<input type=submit name=cmd value=Cancel>");
    puts("</tr>");
    puts("</table>");

    goto exit;
    }

  /*---- set command -----------------------------------------------*/
  if (strcmp(param[1], "Set") == 0)
    {
    status = db_find_key(hDB, 0, dec_path, &hKey);
    if (status != DB_SUCCESS)
      {
      printf("Error: cannot find key %s<P>\n", dec_path);
      goto error;
      }
    db_get_key(hDB, hKey, &key);
    strcpy(data_str, param[2]);
    
    /* get index */
    if (param[3][0])
      index = atoi(param[3]);
    else
      index = 0;

    memset(data, 0, sizeof(data));
    switch (key.type)
      {
      case TID_BYTE:
      case TID_SBYTE:
        data[0] = atoi(data_str); break;
      case TID_CHAR:
        data[0] = data_str[0]; break;
      case TID_WORD:
        *((WORD *) data) = atoi(data_str); break;
      case TID_SHORT:
        *((short int *) data) = atoi(data_str); break;
      case TID_DWORD:
        *((DWORD *) data) = atol(data_str); break;
      case TID_INT:
        *((INT *) data) = atol(data_str); break;
      case TID_BOOL:
        *((BOOL *) data) = atoi(data_str); break;
      case TID_FLOAT:
        *((float *) data) = (float) atof(data_str); break;
      case TID_DOUBLE:
        *((double *) data) = atof(data_str); break;
      case TID_BITFIELD:
        /* TBD */
        break;
      case TID_LINK:
      case TID_STRING:
        strcpy(data, data_str); break;
      }

    /* extend data size for single string if necessary */
    if ((key.type == TID_STRING || key.type == TID_LINK) 
          && (int) strlen(data)+1 > key.item_size &&
        key.num_values == 1)
      key.item_size = strlen(data)+1;

    if (key.item_size == 0)
      key.item_size = rpc_tid_size(key.type);

    if (key.num_values > 1)
      status = db_set_data_index(hDB, hKey, data, key.item_size, index, key.type);
    else
      status = db_set_data(hDB, hKey, data, key.item_size, 1, key.type);

    if (status == DB_NO_ACCESS)
      printf("<h2>Write access not allowed</h2>\n");

    printf("<h2>%s changed successfully</h2>\n\n", dec_path);

    /* strip variable name from path */
    p = dec_path+strlen(dec_path)-1;
    while (*p && *p != '/')
      *p-- = 0;
    if (*p == '/')
      *p = 0;

    strcpy(enc_path, dec_path);
    urlEncode(enc_path);
    }
  
  /*---- find command ----------------------------------------------*/
  if (strcmp(param[1], "Find") == 0)
    {
    /* define hidden fields for current path etc. */
    printf("<input type=hidden name=host value=%s>\n", host_name);
    printf("<input type=hidden name=exp value=\"%s\">\n", exp_name);
    printf("<input type=hidden name=path value=\"%s\">\n", enc_path);

    puts("<table border cellpadding=5>");
    printf("<tr><th colspan=2>Results of search for substring \"%s\"</tr>\n", param[2]);
    printf("<tr><th>Key<th>Value</tr>\n");

    /* start from root */
    db_find_key(hDB, 0, "", &hKey);

    /* scan tree, call "search_callback" for each key */
    db_scan_tree(hDB, hKey, search_callback, (void *) param[2]);

    puts("</table>");
    goto exit;
    }

  
  /*---- cancel command --------------------------------------------*/
  if (strcmp(param[1], "Cancel") == 0)
    {
    printf("<h2>Operation canceled</h2>\n\n");

    /* if path point to a value, strip it */
    status = db_find_key(hDB, 0, dec_path, &hKey);
    if (status != DB_SUCCESS)
      {
      printf("Error: cannot find key %s<P>\n", dec_path);
      goto error;
      }
    db_get_key(hDB, hKey, &key);

    if (key.type != TID_KEY)
      {
      /* strip variable name from path */
      p = dec_path+strlen(dec_path)-1;
      while (*p && *p != '/')
        *p-- = 0;
      if (*p == '/')
        *p = 0;

      strcpy(enc_path, dec_path);
      urlEncode(enc_path);
      }
    }

    
  /*---- print experiment and key/value tables ---------------------*/

  /* title */
  puts("<table border cellpadding=5>");
  puts("<tr><th colspan=2>MIDAS online database<p></tr>");

  /* display buttons */
  if (strcmp(param[1], "Status") == 0)
    {
    puts("<tr><td colspan=2>");
    puts("<input type=submit name=cmd value=Start>");
    puts("<input type=submit name=cmd value=Stop>");
    puts("<input type=submit name=cmd value=ODB>");
    puts("<input type=submit name=cmd value=Help>");
    puts("</tr><p>");
    }
  else
    {
    puts("<tr><td colspan=2>");
    puts("<input type=submit name=cmd value=Start>");
    puts("<input type=submit name=cmd value=Stop>");
    puts("<input type=submit name=cmd value=Search>");
    puts("<input type=submit name=cmd value=Status>");
    puts("<input type=submit name=cmd value=Help>");
    puts("</tr><p>");
    }

  /* display host and experiment */
  if (host_name[0] == 0)
    gethostname(str, 128);
  else
    strcpy(str, host_name);
  printf("<tr><td width=100><a href=\"%s?\">Host:</a><td width=400>%s<br></tr>\n", 
          wwwodb_url, str);

  if (exp_name[0] == 0)
    strcpy(str, "Default");
  else
    strcpy(str, exp_name);
  printf("<tr><td>Experiment:<td>%s<br></tr>\n", str);

  /* define hidden fields for current path etc. */
  printf("<input type=hidden name=host value=%s>\n", host_name);
  printf("<input type=hidden name=exp value=\"%s\">\n", exp_name);
  printf("<input type=hidden name=path value=\"%s\">\n", enc_path);

  /*---- status command --------------------------------------------*/
  if (strcmp(param[1], "Status") == 0)
    {
    show_status_page(hDB);
    goto exit;
    }

  /* display root key */
  printf("<tr><td>Path:<td><pre>");
  printf("<a href=\"%s?host=%s&exp=%s&path=/\">/</a> ", wwwodb_url,
           host_name, exp_name);
  strcpy(path, "/");

  p = dec_path;
  if (*p == '/')
    p++;

  /*---- display path ----*/
  while (*p)
    {
    pd = str;
    while (*p && *p != '/')
      *pd++ = *p++;
    *pd = 0;

    strcat(path, str);
    strcpy(url_path, path);
    urlEncode(url_path);

    printf("<a href=\"%s?host=%s&exp=%s&path=%s\">%s</a> / ", 
           wwwodb_url, host_name , exp_name, url_path, str);

    strcat(path, "/");
    if (*p == '/')
      p++;
    }
  puts("</pre></tr><p>");
  puts("</table>\n\n");

  /* find key via path */
  status = db_find_key(hDB, 0, dec_path, &hKeyRoot);
  if (status != DB_SUCCESS)
    {
    printf("Error: cannot find key %s<P>\n", dec_path);
    goto error;
    }
  
  puts("<table border cellpadding=5>");
  puts("<tr><th width=200>Key<th width=300>Value</tr>");

  /* enumerate subkeys */
  for (i=0 ; ; i++)
    {
    db_enum_key(hDB, hKeyRoot, i, &hKey);
    if (!hKey)
      break;
    db_get_key(hDB, hKey, &key);
    
    strcpy(str, path);
    strcat(str, key.name);
    urlEncode(str);

    if (key.type == TID_KEY || key.type == TID_LINK)
      {
      /* for keys, don't display data value */
      printf("<tr><td><a href=\"%s?host=%s&exp=%s&path=%s\">%s</a><br></tr>\n", 
             wwwodb_url, host_name, exp_name, str, key.name);
      }
    else
      {
      /* display single value */
      if (key.num_values == 1)
        {
        size = sizeof(data);
        db_get_data(hDB, hKey, data, &size, key.type);
        db_sprintf(data_str, data, key.item_size, 0, key.type);

        sprintf(ref, "%s?cmd=modify&host=%s&exp=%s&path=%s", 
                wwwodb_url, host_name, exp_name, str);

        printf("<tr><td>%s<td><a href=\"%s\">%s</a><br></tr>\n", key.name, ref, data_str);
        }
      else
        {
        /* display first value */
        printf("<tr><td rowspan=%d>%s\n", key.num_values, key.name);

        for (j=0 ; j<key.num_values ; j++)
          {
          size = sizeof(data);
          db_get_data(hDB, hKey, data, &size, key.type);
          db_sprintf(data_str, data, key.item_size, j, key.type);

          sprintf(ref, "%s?cmd=modify&index=%d&host=%s&exp=%s&path=%s", 
                  wwwodb_url, j, host_name, exp_name, str);

          if (j>0)
            printf("<tr>");

          printf("<td><a href=\"%s\">[%d] %s</a><br></tr>\n", ref, j, data_str);
          }
        }
      }
    }

  puts("</table>");
  
exit:
error:
  
  /* disconnect from experiment */
  cm_disconnect_experiment();

  puts("</body>");
}

/*------------------------------------------------------------------*/

void show_connect_page()
{
  puts("<html>\n");
  puts("<head>\n");
  puts("<title>MIDAS Online Database <Connect></title>\n");
  puts("</head>\n\n");

  puts("<body>\n");
  printf("<FORM METHOD=\"GET\" ACTION=\"%s\">\n\n", wwwodb_url);

  puts("<center>\n");
  puts("<h1>Welcome to the MIDAS online database</h1>\n\n");

  puts("For some information about MIDAS, consult the ");
  puts("<a href=\"http://pibeta.psi.ch/~pibeta/subprojects/midas/\">MIDAS home page</a>.<p>\n");
  puts("In order to connect to a MIDAS online database, you must specify the\n");
  puts("host name and the experiment name.<br> Leave both fields emtpy to connect\n");
  puts("to the default experiment on the local host:<p>\n\n");

  puts("<table border>\n");
  puts("<tr>\n");
  puts("<td>\n");
  puts("Host name:\n");
  puts("<td>\n");
  puts("<input type=\"text\" size=\"20\" maxlength=\"80\" name=\"host\">\n");
  puts("</tr>\n\n");

  puts("<tr>\n");
  puts("<td>\n");
  puts("Experiment name:\n");
  puts("<td>\n");
  puts("<input type=\"text\" size=\"20\" maxlength=\"80\" name=\"exp\">\n");
  puts("</tr>\n\n");

  puts("</table><p>\n\n");

  puts("<input type=\"submit\" value=\"Connect\">\n");
  puts("</center>\n\n");

  puts("</body>\n");
  puts("</html>\n");
  
}

/*------------------------------------------------------------------*/

void show_help_page()
{
  puts("<head>\n");
  puts("<title>MIDAS Online Database <Help></title>\n");
  puts("</head>\n\n");

  puts("<body>\n");
  puts("<h1>Using the MIDAS Online Database</h1>\n");
  puts("With the MIDAS Online Database (ODB) WWW Gateway you can connect");
  puts("to the database for an experiment running the MIDAS data aqcuisitoin");
  puts("software. The database stores all experiment relevant data like");
  puts("DAQ parameters, initialization information, event and histogram");
  puts("definition etc. and is kept totally in memory for performance");
  puts("reasons.<p>");

  puts("The database is structured hierarchically much like a file system,");
  puts("with subdirectories and single values instead of files. The current");
  puts("position in the hierarchy is called path and displayed in the header");
  puts("together with the currently connected experiment.<p>");

  puts("To descend a hierarchy, click on the key column. You can return");
  puts("to a higher level of the hierarchy by clicking at a level in");
  puts("the path. To modify a value, just click it.<p>");

  puts("<hr>");
  puts("<address>");
  puts("<a href=\"http://pibeta.psi.ch/~stefan\">S. Ritt</a>, 20 Feb 1996");
  puts("</address>");

  puts("</body>\n");
}

/*------------------------------------------------------------------*/

void show_status_page(hDB)
{
int i, size;
char str[128];
char *yn[] = {"No", "Yes"};

  puts("</table>");
  puts("<table border cellpadding=5>");
  puts("<tr><th colspan=4>Status</tr>");

  /* display run state */
  size = sizeof(i);
  printf("<tr><td colspan=2>State: <b> ");
  i = STATE_STOPPED; /* default */
  db_get_value(hDB, 0, "/Runinfo/State", &i, &size, TID_INT);
  switch (i)
    {
    case STATE_STOPPED: printf("Stopped"); break;
    case STATE_RUNNING: printf("Running"); break;
    case STATE_PAUSED: printf("Paused"); break;
    default: printf("Unknown");
    }
  i = 0;
  db_get_value(hDB, 0, "/Runinfo/Online state", &i, &size, TID_INT);
  if (i==1)
    puts(",Online</b>");
  else
    puts(",Offline</b>");

  /* and run number */
  i = 1;
  db_get_value(hDB, 0, "/Runinfo/Run number", &i, &size, TID_INT);
  printf("<td colspan=2>Run number: <b>%d</b></tr>", i);

  /* start time */
  size = sizeof(str);
  str[0] = 0;
  db_get_value(hDB, 0, "/Runinfo/Start time", str, &size, TID_STRING);  
  printf("<tr><td colspan=4>Run started at: <b>%s</b></tr>", str);
  
  /* flags */
  puts("<tr><th colspan=4>Flags</tr>");
  size = sizeof(i);
  i = 0;
  db_get_value(hDB, 0, "/Runinfo/Flags/Analyze all", &i, &size, TID_BOOL);  
  printf("<tr><td colspan=2>Analyze all events: <b>%s</b>", yn[i]);
  db_get_value(hDB, 0, "/Runinfo/Flags/Log history", &i, &size, TID_BOOL);  
  printf("<td colspan=2>Log history: <b>%s</b>\n", yn[i]);

  db_get_value(hDB, 0, "/Runinfo/Flags/Save histos", &i, &size, TID_BOOL);  
  printf("<tr><td>Save histos: <b>%s</b>", yn[i]);
  db_get_value(hDB, 0, "/Runinfo/Flags/Save 2D histos", &i, &size, TID_BOOL);  
  printf("<td>Save 2D histos: <b>%s</b>", yn[i]);
  db_get_value(hDB, 0, "/Runinfo/Flags/Save odb", &i, &size, TID_BOOL);  
  printf("<td>Save database: <b>%s</b></tr>", yn[i]);
  db_get_value(hDB, 0, "/Runinfo/Flags/Print runsheet", &i, &size, TID_BOOL);  
  printf("<td>Print runsheet: <b>%s</b></tr>\n", yn[i]);

  /* logger */
  puts("<tr><th colspan=4>Logger</tr>");
  db_get_value(hDB, 0, "/Runinfo/Online/Stop after events", &i, &size, TID_INT);
  printf("<tr><td colspan=2>Stop after : <b>%d</b> events</b>", i);
  db_get_value(hDB, 0, "/Runinfo/Online/Auto restart", &i, &size, TID_BOOL);
  printf("<td colspan=2>Auto restart: <b>%s</b></tr>\n", yn[i]);  

  puts("</table>");
}

/*------------------------------------------------------------------*/

main(int argc, char *argv[]) 
{
int  i;
char *p, *pitem;
char *pRequestMethod;
char InputBuffer[4096];
char param[6][128];
char str[128];

  /* change to directory specified by DIR */
#ifdef WDIR
  strcpy(str, WDIR); 
  chdir(str);
#endif

  for (i=0 ; i<5 ; i++)
    param[i][0] = 0;

  /* set dummys for environment variables for testing */
  if (getenv("SERVER_NAME") == NULL)
    {
    putenv("SERVER_NAME=pierre");
    putenv("SCRIPT_NAME=wwwodb.exe");
    putenv("REQUEST_METHOD=GET");
    putenv("QUERY_STRING=host=&exp=&path=%2FRuninfo");
    }

  /* assemble URL to wwwodb */
  sprintf(wwwodb_url, "http://%s", getenv("SERVER_NAME"));
  strcpy(str, getenv("SCRIPT_NAME"));
  if (str[0] != '/')
    strcat(wwwodb_url, "/");
  strcat(wwwodb_url, str);

#ifdef OS_WINNT
  /* change '\' to '/' under NT */
  while (strchr(wwwodb_url, '\\'))
    *strchr(wwwodb_url, '\\') = '/';
#endif
  
  /* Tell the client what we're going to send */
  printf("Content-type: text/html\n\n");

  /* What method were we invoked through? */
  pRequestMethod = getenv("REQUEST_METHOD");
  if (pRequestMethod==NULL) 
    {
    printf("Error: REQUEST_METHOD not specified\n");
    return 0;
    }

  if (strcmp(pRequestMethod,"GET")==0) 
    {
    /* Parse the data in the search term */
    p = getenv("QUERY_STRING");
    i = 0;
    if (p!=NULL) 
      {
      strncpy(InputBuffer,p,sizeof(InputBuffer));
      p = strtok(InputBuffer,"&");
      while (p!=NULL) 
        {
        pitem=p;
        p = strchr(p,'=');
        if (p != NULL)
          {
          *p++ = 0;
          urlDecode(pitem);
          if (strcmp(pitem, "value") == 0)
            {
            /* convert '+' to ' ' */
            while (strchr(p, '+'))
              *strchr(p, '+') = ' ';
            }

          urlDecode(p);

          if (strcmp(pitem, "host") == 0)
            strcpy(host_name, p);
          if (strcmp(pitem, "exp") == 0)
            strcpy(exp_name, p);

          if (strcmp(pitem, "path") == 0)
            strcpy(param[0], p);
          if (strcmp(pitem, "cmd") == 0)
            strcpy(param[1], p);
          if (strcmp(pitem, "value") == 0)
            strcpy(param[2], p);
          if (strcmp(pitem, "index") == 0)
            strcpy(param[3], p);

          p = strtok(NULL,"&");
          }
        }

      /* print connect page of no params */
      if (!InputBuffer[0])
        show_connect_page();
      else
      /* call odb interpreter */
        interprete(param);
    
      }
    else
      {
      printf("Error: No QUERY_STRING specified\n");
      return 0;
      }
    }
  else
    {
    printf("Error: Request method %s not supported\n", pRequestMethod);
    return 0;
    }

  return 0;
}

